<?php


/**
 *
 * Some collection macros to add extended functionality to base collection
 *
 */

use Illuminate\Http\Request;
use Illuminate\Support\Collection;

if (!Collection::hasMacro('dd')) {
    /*
     * Dump the contents of the collection and terminate the script.
     */
    Collection::macro('dd', function () {
        dd($this);
    });
}

if (!Collection::hasMacro('dump')) {
    /*
     * Dump the arguments given followed by the collection.
     */
    Collection::macro('dump', function () {
        Collection::make(func_get_args())
            ->push($this)
            ->each(function ($item) {
                (new Dumper)->dump($item);
            });
        return $this;
    });
}

if (!Collection::hasMacro('collect')) {
    /*
     * Get a new collection from the collection by key.
     *
     * @param  mixed  $key
     * @param  mixed  $default
     *
     * @return static
     */
    Collection::macro('collect', function ($key, $default = null) {
        return new static($this->get($key, $default));
    });
}


if (!Collection::hasMacro('ifEmpty')) {
    /*
     * Execute a callable if the collection is empty, then return the collection.
     *
     * @param callable $callback
     *
     * @return \Illuminate\Support\Collection
     */
    Collection::macro('ifEmpty', function (callable $callback) {
        if ($this->isEmpty()) {
            $callback($this);
        }
        return $this;
    });
}


if (!Collection::hasMacro('fill')) {
    Collection::macro('fill', function ($value) {
        return $this->combine(array_fill(0, $this->count(), $value));
    });
}

if (!Collection::hasMacro('transpose')) {
    Collection::macro('transpose', function () {
        $items = array_map(function (...$items) {
            return $items;
        }, ...$this->values());
        return new static($items);
    });
}

if (!Collection::hasMacro('transposeWithKeys')) {
    Collection::macro('transposeWithKeys', function () {
        $items = array_map(function (...$items) {
            return array_combine($this->keys()->toArray(), $items);
        }, ...$this->values());
        return new static(array_combine(array_keys($this->first()), $items));
    });
}

if (!Collection::hasMacro('resetChunkArrayKeys')) {
    Collection::macro('resetChunkArrayKeys', function () {
        return $this->map(function ($chunk) {
            return $chunk->values();
        });
    });
}

if (!Collection::hasMacro('videoIds')) {
    Collection::macro('intersectKey', function ($data) {
        if ($data instanceof static) {
            $data = $data->toArray();
        }
        return new static(array_intersect_key($this->items, $data));
    });
}

if (!Collection::hasMacro('getFileName')) {
    Collection::macro('getFileName', function ($fieldName, $key) {
        return $this->get($key, function () use ($fieldName, $key) {
            return array_get("oldImages.{$fieldName}.{$key}", $this->input());
        });
    });
}


/**
 *
 * Some Request macros to add extended functionality to base request
 *
 */

if (!Request::hasMacro('processStoreFile')) {
    /**
     * Store the attach file if one is present.
     *
     * @param string $fileName The field name of the image
     * @param string $location The location to store the image
     * @return null|string
     */
    Request::macro('processStoreFile', function ($fileName, $location) {

        if (!$this->hasFile($fileName)) {
            return null;
        }
        $file = $this->file($fileName);

        $hash = $file->hashName();
        $file->storeAs('public/' . $location, $hash);
        return $hash;
    });
}


if (!Request::hasMacro('storeFile')) {

    /**
     * Store the attach file if one is present.
     *
     * @param string $fileName The field name of the image
     * @param string $location The location to store the image
     * @return void
     */
    Request::macro('storeFile', function ($fileName, $location) {

        $hash = $this->processStoreFile($fileName, $location);

        if (!is_null($hash)) {
            $this->merge([$fileName => $hash]);
        }
    });
}


if (!Request::hasMacro('storeMultiInputImages')) {

    /**
     * Store multiple uploaded images.
     *
     * @param string $fieldName
     * @param string $location
     * @return void
     */
    Request::macro('storeMultiInputImages', function ($fieldName, $location) {

        if (!$this->has($fieldName)) {
            return;
        }

        $fields = collect($this->get($fieldName));

        // Store each uploaded image and return a Collection of hashed image names.
        $hashedFileNames = $fields->map(function ($value, $key) use ($fieldName, $location) {
            $fileName = "{$fieldName}.{$key}.image";
            return $this->processStoreFile($fileName, $location);
        })->filter();


        // Update the data to include the filename, this is either the new hashed filename
        // or the old image file name.
        $newData = $fields
            ->map(function ($value, $key) use ($hashedFileNames, $fieldName) {
                return array_merge($value, [
                    'image' => $hashedFileNames->getFileName($fieldName, $key),
                ]);
            })
            ->filter()
            ->toArray();

        $this->merge([$fieldName => $newData]);
    });
}


if (!Request::hasMacro('cleanEmptyMultiInput')) {

    /**
     * Removes any empty multi input data.
     *
     * @param array $removable The fields to clean
     * @param boolean $removeEmptyValues Should empty sub elements be removed
     * @return void
     * @SuppressWarnings(PHPMD)
     */
    Request::macro('cleanEmptyMultiInput', function (array $removable, $removeEmptyValues = true) {
        collect($removable)
            ->each(function ($key) use ($removeEmptyValues) {
                $data = collect($this->get($key))
                    ->map(function ($data) use ($removeEmptyValues) {
                        //If the data is not an array i.e a standard MultiInput
                        if (!is_array($data)) {
                            return $data === '' ? null : $data;
                        }
                        return $removeEmptyValues ? array_filter($data) : $data;
                    })
                    ->reject(function ($item) {
                        return empty($item) || (is_array($item) && array_key_exists('image', $item) && $item['image'] == 'undefined');
                    })
                    ->toArray();

                if (empty($data)) {
                    $data = null;
                }
                $this->merge([$key => $data]);
            });
    });

}
